#include "defs.h"
#include <stdio.h>
#include "imageio.h"
#include "imagemath.h"
#include "sectorlist.h"
#ifdef USE_DEBUG
#include "time.h"
#endif

struct sectorlist* checkmove(ImagePtr *dest, ImagePtr imsource, int deltabright, int sectorsize) {
/*Returns a list of overbrighted sectors in source,
  and paints this sectors into a destination image
  a sector is overbrighted when more than 20% over
  the average sector brightness + deltabright
  */
   ImagePtr imdest;		/*Pointer to the dest-image*/
   struct sectorlist* answer;	/*Sectorlist to return*/
   int t,tt,actual,avg;		/*Several integers*/
   int x,y;
   int color[256];		/*Colormap*/

#ifdef USE_DEBUG
   clock_t starttime;
   starttime=clock();
   fprintf(stderr,"Running movement check with threshold %i and sectorsize %i...\n",deltabright,sectorsize);
#endif

   imdest = createim(widthim(imsource),heightim(imsource));
   *dest=imdest;
   /*We create a destination image the size of the source*/
   /*and give its adress back*/

   t=0;
   while (t<256) {
      color[t]=collorallocateim(imdest,t,t,t);
      t++;
   }
   /*We create some sort of 256 colors b/w colormap for the destination*/

   t=0;
   avg=0;
   x=xsectorsim(imdest,sectorsize);
   y=ysectorsim(imdest,sectorsize);
   while (t<x) {
      tt=0;
      while (tt<y) {
         actual=scalepixelim(imsource,t*sectorsize*256,tt*sectorsize*256,
	 	(t+1)*sectorsize*256,(tt+1)*sectorsize*256);
	 avg+=actual;
	 tt++;
      }
      t++;
   }
   avg=avg/(x*y);
   /*Ok The important stuff, part 1
     Since i am too lacy to save the whole "sectormap" in a dynamically allocated structure, i go
     through the image 2 times.
     This is the first times, going over every 10x10 pixel sector and adding its average brightness to sum
     then we divide the overall sum through the sector count to get the average sector brightness*/

   /*Now comes the new 0.0.2 part:
     We create an object of type sectorlist, and save all overbrighted sectors there!*/
   answer=sectorlist_new();

   t=0;
   while (t<x) {
      tt=0;
      while (tt<y) {
         actual=scalepixelim(imsource,t*sectorsize*256,tt*sectorsize*256,
	 	(t+1)*sectorsize*256,(tt+1)*sectorsize*256);
         if (actual>(((120*avg)/100)+deltabright)) {
	    sectorlist_addsector(answer,t,tt,actual);
	 }
	 drawsectorim (imdest,t,tt,color[actual],sectorsize);
         tt++;
      }
      t++;
   }
   /*Important stuff, part 2
     Now, we go through the sectors again, comparing each sector to the average
     If a sector is more than deltabright + the average we are most likely
     to have movement in this sector. So we store that sector in a list.
     The "average" somehow represents the background cripple created by our camera.
     It should be almost the same in every sector. being deltabright+20% brighter this enlightened sector
     is definitely strange, and should be counted as possible movement
   */

#ifdef USE_DEBUG
   fprintf(stderr," ... took  %.4f seconds!\n",(1.0 * (clock() - starttime) / CLOCKS_PER_SEC));
#endif
   return answer;
   /*We give back the sectorlist!*/
}


